/**************************************************************************\
*
* All files in GdiPlusLib have been modified from the Microsoft
* GdiPlus C++ header files.
*
* Module Name:
*
*   GdiPlusBitmap.h
*
* Abstract:
*
*   Bitmap related declarations
*
\**************************************************************************/

#ifndef _GDIPLUSBITMAP_H
#define _GDIPLUSBITMAP_H

#include <objidl.h>
#include "GdiPlusEnums.h"
#include "GdiPlusTypes.h"
#include "GdiPlusStubs.h"
#include "GdiPlusImaging.h"

typedef struct tagGp_Image
{
	GP_GPIMAGE * nativeImage;
	GpStatus lastResult;
}GP_IMAGE;

typedef struct tagGp_Bitmap
{
	GP_GPBITMAP * nativeImage;
	GpStatus lastResult;
}GP_BITMAP;


//----------------------------------------------------------------------------
// Bitmap methods API calls
//----------------------------------------------------------------------------
GpStatus _stdcall GdipCreateBitmapFromStream(IStream * stream, GP_GPBITMAP **bitmap);
GpStatus _stdcall GdipCreateBitmapFromFile(const WCHAR* filename, GP_GPBITMAP **bitmap);
GpStatus _stdcall GdipCreateBitmapFromStreamICM(IStream* stream, GP_GPBITMAP **bitmap);
GpStatus _stdcall GdipCreateBitmapFromFileICM(const WCHAR* filename, GP_GPBITMAP **bitmap);
GpStatus _stdcall GdipCreateBitmapFromScan0(INT width, INT height, INT stride,
                          GpPixelFormat format, BYTE* scan0, GP_GPBITMAP** bitmap);
GpStatus _stdcall GdipCreateBitmapFromGraphics(INT width, INT height,
                             GP_GPGRAPHICS* target, GP_GPBITMAP** bitmap);
GpStatus _stdcall GdipCreateBitmapFromDirectDrawSurface(IDirectDrawSurface7 * surface, GP_GPBITMAP** bitmap);
GpStatus _stdcall GdipCreateBitmapFromGdiDib(const BITMAPINFO* gdiBitmapInfo, VOID* gdiBitmapData,
                           GP_GPBITMAP** bitmap);
GpStatus _stdcall GdipCreateBitmapFromHBITMAP(HBITMAP hbm, HPALETTE hpal, GP_GPBITMAP** bitmap);
GpStatus _stdcall GdipCreateHBITMAPFromBitmap(GP_GPBITMAP* bitmap, HBITMAP* hbmReturn, ARGB background);
GpStatus _stdcall GdipCreateBitmapFromHICON(HICON hicon, GP_GPBITMAP** bitmap);
GpStatus _stdcall GdipCreateHICONFromBitmap(GP_GPBITMAP* bitmap, HICON* hbmReturn);
GpStatus _stdcall GdipCreateBitmapFromResource(HINSTANCE hInstance, const WCHAR* lpBitmapName,
                    GP_GPBITMAP** bitmap);
GpStatus _stdcall GdipCloneBitmapArea(REAL x, REAL y, REAL width, REAL height,
					GpPixelFormat format, GP_GPBITMAP *srcBitmap, GP_GPBITMAP **dstBitmap);

GpStatus _stdcall GdipCloneBitmapAreaI(INT x, INT y, INT width, INT height,
                     GpPixelFormat format, GP_GPBITMAP *srcBitmap, GP_GPBITMAP **dstBitmap);
GpStatus _stdcall GdipBitmapLockBits(GP_GPBITMAP* bitmap, const GPRECTI* rect,
                   UINT flags, GpPixelFormat format, BITMAPDATA* lockedBitmapData);
GpStatus _stdcall GdipBitmapUnlockBits(GP_GPBITMAP* bitmap, BITMAPDATA* lockedBitmapData);
GpStatus _stdcall GdipBitmapGetPixel(GP_GPBITMAP* bitmap, INT x, INT y, ARGB *color);
GpStatus _stdcall GdipBitmapSetPixel(GP_GPBITMAP* bitmap, INT x, INT y, ARGB color);
GpStatus _stdcall GdipBitmapSetResolution(GP_GPBITMAP* bitmap, REAL xdpi, REAL ydpi);

//----------------------------------------------------------------------------
// Image methods
//----------------------------------------------------------------------------
GpStatus _stdcall GdipLoadImageFromStream(IStream* stream, GP_GPIMAGE **image);
GpStatus _stdcall GdipLoadImageFromFile(const WCHAR* filename, GP_GPIMAGE **image);
GpStatus _stdcall GdipLoadImageFromStreamICM(IStream* stream, GP_GPIMAGE **image);
GpStatus _stdcall GdipLoadImageFromFileICM(const WCHAR* filename, GP_GPIMAGE **image);
GpStatus _stdcall GdipCloneImage(GP_GPIMAGE *image, GP_GPIMAGE **cloneImage);
GpStatus _stdcall GdipDisposeImage(GP_GPIMAGE *image);
GpStatus _stdcall GdipSaveImageToFile(GP_GPIMAGE *image, const WCHAR* filename,
                    const CLSID* clsidEncoder, const ENCODERPARAMETERS* encoderParams);
GpStatus _stdcall GdipSaveImageToStream(GP_GPIMAGE *image, IStream* stream,
                      const CLSID* clsidEncoder, const ENCODERPARAMETERS* encoderParams);
GpStatus _stdcall GdipSaveAdd(GP_GPIMAGE *image, const ENCODERPARAMETERS* encoderParams);
GpStatus _stdcall GdipSaveAddImage(GP_GPIMAGE *image, GP_GPIMAGE* newImage,
                 const ENCODERPARAMETERS* encoderParams);
GpStatus _stdcall GdipGetImageGraphicsContext(GP_GPIMAGE *image, GP_GPGRAPHICS **graphics);
GpStatus _stdcall GdipGetImageBounds(GP_GPIMAGE *image, GPRECTF *srcRect, GpUnit *srcUnit);
GpStatus _stdcall GdipGetImageDimension(GP_GPIMAGE *image, REAL *width, REAL *height);
GpStatus _stdcall GdipGetImageType(GP_GPIMAGE *image, GpImageType *type);
GpStatus _stdcall GdipGetImageWidth(GP_GPIMAGE *image, UINT *width);
GpStatus _stdcall GdipGetImageHeight(GP_GPIMAGE *image, UINT *height);
GpStatus _stdcall GdipGetImageHorizontalResolution(GP_GPIMAGE *image, REAL *resolution);
GpStatus _stdcall GdipGetImageVerticalResolution(GP_GPIMAGE *image, REAL *resolution);
GpStatus _stdcall GdipGetImageFlags(GP_GPIMAGE *image, UINT *flags);
GpStatus _stdcall GdipGetImageRawFormat(GP_GPIMAGE *image, GUID *format);
GpStatus _stdcall GdipGetImagePixelFormat(GP_GPIMAGE *image, GpPixelFormat *format);
GpStatus _stdcall GdipGetImageThumbnail(GP_GPIMAGE *image, UINT thumbWidth, UINT thumbHeight,
                      GP_GPIMAGE **thumbImage,
                      GetThumbnailImageAbort callback, VOID * callbackData);
GpStatus _stdcall GdipGetEncoderParameterListSize(GP_GPIMAGE *image, const CLSID* clsidEncoder,
                                UINT* size);
GpStatus _stdcall GdipGetEncoderParameterList(GP_GPIMAGE *image, const CLSID* clsidEncoder,
                            UINT size, ENCODERPARAMETERS* buffer);
GpStatus _stdcall GdipImageGetFrameDimensionsCount(GP_GPIMAGE* image, UINT* count);
GpStatus _stdcall GdipImageGetFrameDimensionsList(GP_GPIMAGE* image, GUID* dimensionIDs, UINT count);
GpStatus _stdcall GdipImageGetFrameCount(GP_GPIMAGE *image, const GUID* dimensionID, UINT* count);
GpStatus _stdcall GdipImageSelectActiveFrame(GP_GPIMAGE *image, const GUID* dimensionID,
                           UINT frameIndex);
GpStatus _stdcall GdipImageRotateFlip(GP_GPIMAGE *image, GpRotateFlipType rfType);
GpStatus _stdcall GdipGetImagePalette(GP_GPIMAGE *image, COLORPALETTE *palette, INT size);
GpStatus _stdcall GdipSetImagePalette(GP_GPIMAGE *image, const COLORPALETTE *palette);
GpStatus _stdcall GdipGetImagePaletteSize(GP_GPIMAGE *image, INT *size);
GpStatus _stdcall GdipGetPropertyCount(GP_GPIMAGE *image, UINT* numOfProperty);
GpStatus _stdcall GdipGetPropertyIdList(GP_GPIMAGE *image, UINT numOfProperty, PROPID* list);
GpStatus _stdcall GdipGetPropertyItemSize(GP_GPIMAGE *image, PROPID propId, UINT* size);
GpStatus _stdcall GdipGetPropertyItem(GP_GPIMAGE *image, PROPID propId,UINT propSize,
                    PROPERTYITEM* buffer);
GpStatus _stdcall GdipGetPropertySize(GP_GPIMAGE *image, UINT* totalBufferSize, UINT* numProperties);
GpStatus _stdcall GdipGetAllPropertyItems(GP_GPIMAGE *image, UINT totalBufferSize,
                        UINT numProperties, PROPERTYITEM* allItems);
GpStatus _stdcall GdipRemovePropertyItem(GP_GPIMAGE *image, PROPID propId);
GpStatus _stdcall GdipSetPropertyItem(GP_GPIMAGE *image, const PROPERTYITEM* item);
GpStatus _stdcall GdipImageForceValidation(GP_GPIMAGE *image);

//----------------------------------------------------------------------------
// GdiPlusLib.lib functions
//----------------------------------------------------------------------------
GP_IMAGE GpImage_FromFile(const WCHAR * filename, BOOL useEmbeddedColorManagement);
GP_IMAGE GpImage_FromIStream(IStream * stream, BOOL useEmbeddedColorManagement);
VOID     GpImage_Delete(GP_IMAGE * this);
GP_IMAGE GpImage_Clone(GP_IMAGE * this);
UINT     GpImage_GetEncoderParameterListSize(GP_IMAGE * this, const CLSID * clsidEncoder);
GpStatus GpImage_GetEncoderParameterList(GP_IMAGE * this, const CLSID * clsidEncoder, UINT size, ENCODERPARAMETERS * buffer);
GpStatus GpImage_SaveFile(GP_IMAGE * this, const WCHAR * filename, const CLSID * clsidEncoder, const ENCODERPARAMETERS * encoderParams);
GpStatus GpImage_SaveStream(GP_IMAGE * this, IStream * stream, const CLSID * clsidEncoder, const ENCODERPARAMETERS * encoderParams);
GpStatus GpImage_SaveAdd(GP_IMAGE * this, const ENCODERPARAMETERS * encoderParams);
GpStatus GpImage_SaveAddImage(GP_IMAGE * this, GP_IMAGE * newImage, const ENCODERPARAMETERS * encoderParams);
GpImageType GpImage_GetType(GP_IMAGE * this);
GpStatus GpImage_GetPhysicalDimension(GP_IMAGE * this, SIZEF * size);
GpStatus GpImage_GetBounds(GP_IMAGE * this, RECTF * srcRect, GpUnit * srcUnit);
UINT     GpImage_GetWidth(GP_IMAGE * this);
UINT     GpImage_GetHeight(GP_IMAGE * this);
REAL     GpImage_GetHorizontalResolution(GP_IMAGE * this);
REAL     GpImage_GetVerticalResolution(GP_IMAGE * this);
UINT     GpImage_GetFlags(GP_IMAGE * this);
GpStatus GpImage_GetRawFormat(GP_IMAGE * this, GUID * format);
GpPixelFormat GpImage_GetPixelFormat(GP_IMAGE * this);
INT      GpImage_GetPaletteSize(GP_IMAGE * this);
GpStatus GpImage_GetPalette(GP_IMAGE * this, COLORPALETTE * palette, INT size);
GpStatus GpImage_SetPalette(GP_IMAGE * this, const COLORPALETTE * palette);
GP_IMAGE GpImage_GetThumbnailImage(GP_IMAGE * this, UINT thumbWidth, UINT thumbHeight, GetThumbnailImageAbort callback, VOID * callbackData);
UINT     GpImage_GetFrameDimensionsCount(GP_IMAGE * this);
GpStatus GpImage_GetFrameDimensionsList(GP_IMAGE * this, GUID * dimensionIDs, UINT count);
UINT     GpImage_GetFrameCount(GP_IMAGE * this, const GUID * dimensionID);
GpStatus GpImage_SelectActiveFrame(GP_IMAGE * this, const GUID * dimensionID, UINT frameIndex);
GpStatus GpImage_RotateFlip(GP_IMAGE * this, GpRotateFlipType rotateFlipType);
UINT     GpImage_GetPropertyCount(GP_IMAGE * this);
GpStatus GpImage_GetPropertyIdList(GP_IMAGE * this, UINT numOfProperty, PROPID * list);
UINT     GpImage_GetPropertyItemSize(GP_IMAGE * this, PROPID propId);
GpStatus GpImage_GetPropertyItem(GP_IMAGE * this, PROPID propId, UINT propSize, PROPERTYITEM * buffer);
GpStatus GpImage_GetPropertySize(GP_IMAGE * this, UINT * totalBufferSize, UINT * numProperties);
GpStatus GpImage_GetAllPropertyItems(GP_IMAGE * this, UINT totalBufferSize, UINT numProperties, PROPERTYITEM * allItems);
GpStatus GpImage_RemovePropertyItem(GP_IMAGE * this, PROPID propId);
GpStatus GpImage_SetPropertyItem(GP_IMAGE * this, const PROPERTYITEM * item);
GpStatus GpImage_GetLastStatus(GP_IMAGE * this);
GP_IMAGE  GpImage_Image(GP_GPIMAGE * nativeImage, GpStatus status);
GP_BITMAP GpBitmap_Stream(IStream * stream, BOOL useEmbeddedColorManagement);
GP_BITMAP GpBitmap_Bytes(INT width, INT height, INT stride, GpPixelFormat format, BYTE * scan0);
GP_BITMAP GpBitmap_PixFmt(INT width, INT height, GpPixelFormat format);
GP_BITMAP GpBitmap_Graphics(INT width, INT height, GP_GRAPHICS * target);
GP_BITMAP GpBitmap_DDraw(IDirectDrawSurface7 * surface);
GP_BITMAP GpBitmap_BITMAPINFO(const BITMAPINFO * gdiBitmapInfo, VOID * gdiBitmapData);
GP_BITMAP GpBitmap_Pal(HBITMAP hbm, HPALETTE hpal);
GP_BITMAP GpBitmap_Icon(HICON hicon);
GP_BITMAP GpBitmap_Resource(HINSTANCE hInstance, const WCHAR * bitmapName);
GP_BITMAP GpBitmap_FromFile(const WCHAR * filename, BOOL useEmbeddedColorManagement);
GP_BITMAP GpBitmap_FromStream(IStream * stream, BOOL useEmbeddedColorManagement);
GP_BITMAP GpBitmap_FromDirectDrawSurface7(IDirectDrawSurface7 * surface);
GP_BITMAP GpBitmap_FromBITMAPINFO(const BITMAPINFO * gdiBitmapInfo, VOID * gdiBitmapData);
GP_BITMAP GpBitmap_FromHBITMAP(HBITMAP hbm, HPALETTE hpal);
GP_BITMAP GpBitmap_FromHICON(HICON hicon);
GP_BITMAP GpBitmap_FromResource(HINSTANCE hInstance, const WCHAR * bitmapName);
GpStatus  GpBitmap_GetHBITMAP(GP_BITMAP * this, const COLOR * colorBackground, HBITMAP * hbmReturn);
GpStatus  GpBitmap_GetHICON(GP_BITMAP * this, HICON * hiconReturn);
GP_BITMAP GpBitmap_Bitmap(GP_GPBITMAP * nativeBitmap);
GP_BITMAP GpBitmap_Clone4I(GP_BITMAP * this, INT x, INT y, INT width, INT height, GpPixelFormat format);
GP_BITMAP GpBitmap_ClonePixFmt(const RECTI * rect, GpPixelFormat format);
GP_BITMAP GpBitmap_Clone4F(GP_BITMAP * this, REAL x, REAL y, REAL width, REAL height, GpPixelFormat format);
GpStatus  GpBitmap_LockBits(GP_BITMAP * this, const RECTI * rect, UINT flags, GpPixelFormat format, BITMAPDATA * lockedBitmapData);
GpStatus  GpBitmap_UnlockBits(GP_BITMAP * this, BITMAPDATA * lockedBitmapData);
GpStatus  GpBitmap_GetPixel(GP_BITMAP * this, INT x, INT y, COLOR * color);
GpStatus  GpBitmap_SetPixel(GP_BITMAP * this, INT x, INT y, const COLOR * color);
GpStatus  GpBitmap_SetResolution(GP_BITMAP * this, REAL xdpi, REAL ydpi);

#endif // _GDIPLUSBITMAP_H
