/**************************************************************************\
*
* All files in GdiPlusLib have been modified from the Microsoft
* GdiPlus C++ header files.
*
* Module Name:
*
*   GdiplusEnums.h
*
* Abstract:
*
*   Various enumeration types
*
\**************************************************************************/

#ifndef _GDIPLUSENUMS_H
#define _GDIPLUSENUMS_H

#define WIN32_LEAN_AND_MEAN  /* speed up compilations */
#include <Windows.h>
#include <WinGDI.h>

typedef int BOOL;
//--------------------------------------------------------------------------
// Default bezier flattening tolerance in device pixels.
//--------------------------------------------------------------------------
#define FLATNESSDEFAULT 1.0f/4.0f;
//--------------------------------------------------------------------------
// Graphics and Container State cookies
//--------------------------------------------------------------------------

typedef unsigned int GpGraphicsState;
typedef unsigned int GpGraphicsContainer;

typedef enum tagStatus
{
    eOk 				= 0,
    eGenericError 		= 1,
    eInvalidParameter 	= 2,
    eOutOfMemory 		= 3,
    eObjectBusy 		= 4,
    eInsufficientBuffer = 5,
    eNotImplemented 	= 6,
    eWin32Error 		= 7,
    eWrongState 		= 8,
    eAborted 			= 9,
    eFileNotFound 		= 10,
    eValueOverflow 		= 11,
    eAccessDenied 		= 12,
    eUnknownImageFormat = 13,
    eFontFamilyNotFound = 14,
    eFontStyleNotFound  = 15,
    eNotTrueTypeFont 	= 16,
    eUnsupportedGplusVersion  = 17,
    eGplusNotInitialized 		= 18
}GpStatus;

//--------------------------------------------------------------------------
// Fill mode constants
//--------------------------------------------------------------------------

typedef enum tagFillMode
{
    eFillModeAlternate,        // 0
    eFillModeWinding           // 1
}GpFillMode;

//--------------------------------------------------------------------------
// Quality mode constants
//--------------------------------------------------------------------------

typedef enum tagQualityMode
{
    eQualityModeInvalid   = -1,
    eQualityModeDefault   = 0,
    eQualityModeLow       = 1, // for apps that need the best performance
    eQualityModeHigh      = 2  // for apps that need the best rendering quality
}GpQualityMode;

//--------------------------------------------------------------------------
// Alpha compositing mode constants
//--------------------------------------------------------------------------

typedef enum tagCompositingMode
{
    eCompositingModeSourceOver,    // 0
    eCompositingModeSourceCopy     // 1
}GpCompositingMode;

//--------------------------------------------------------------------------
// Alpha compositing quality constants
//--------------------------------------------------------------------------

typedef enum tagCompositingQuality
{
    eCompositingQualityInvalid          = eQualityModeInvalid,
    eCompositingQualityDefault          = eQualityModeDefault,
    eCompositingQualityHighSpeed        = eQualityModeLow,
    eCompositingQualityHighQuality      = eQualityModeHigh,
    eCompositingQualityGammaCorrected,
    eCompositingQualityAssumeLinear
}GpCompositingQuality;

//--------------------------------------------------------------------------
// Unit constants
//--------------------------------------------------------------------------

typedef enum tagUnit
{
    eUnitWorld,      // 0 -- World coordinate (non-physical unit)
    eUnitDisplay,    // 1 -- Variable -- for PageTransform only
    eUnitPixel,      // 2 -- Each unit is one device pixel.
    eUnitPoint,      // 3 -- Each unit is a printer's point, or 1/72 inch.
    eUnitInch,       // 4 -- Each unit is 1 inch.
    eUnitDocument,   // 5 -- Each unit is 1/300 inch.
    eUnitMillimeter  // 6 -- Each unit is 1 millimeter.
}GpUnit;

//--------------------------------------------------------------------------
// MetafileFrameUnit
//
// The frameRect for creating a metafile can be specified in any of these
// units.  There is an extra frame unit value (MetafileFrameUnitGdi) so
// that units can be supplied in the same units that GDI expects for
// frame rects -- these units are in .01 (1/100ths) millimeter units
// as defined by GDI.
//--------------------------------------------------------------------------
typedef enum tagMetafileFrameUnit
{
    eMetafileFrameUnitPixel      = eUnitPixel,
    eMetafileFrameUnitPoint      = eUnitPoint,
    eMetafileFrameUnitInch       = eUnitInch,
    eMetafileFrameUnitDocument   = eUnitDocument,
    eMetafileFrameUnitMillimeter = eUnitMillimeter,
    eMetafileFrameUnitGdi                        // GDI compatible .01 MM units
}GpMetafileFrameUnit;

//--------------------------------------------------------------------------
// Coordinate space identifiers
//--------------------------------------------------------------------------

typedef enum tagCoordinateSpace
{
    eCoordinateSpaceWorld,     // 0
    eCoordinateSpacePage,      // 1
    eCoordinateSpaceDevice     // 2
}GpCoordinateSpace;

//--------------------------------------------------------------------------
// Various wrap modes for brushes
//--------------------------------------------------------------------------

typedef enum tagWrapMode
{
    eWrapModeTile,        // 0
    eWrapModeTileFlipX,   // 1
    eWrapModeTileFlipY,   // 2
    eWrapModeTileFlipXY,  // 3
    eWrapModeClamp        // 4
}GpWrapMode;

//--------------------------------------------------------------------------
// Various hatch styles
//--------------------------------------------------------------------------

typedef enum tagHatchStyle
{
    eHatchStyleHorizontal,                   // 0
    eHatchStyleVertical,                     // 1
    eHatchStyleForwardDiagonal,              // 2
    eHatchStyleBackwardDiagonal,             // 3
    eHatchStyleCross,                        // 4
    eHatchStyleDiagonalCross,                // 5
    eHatchStyle05Percent,                    // 6
    eHatchStyle10Percent,                    // 7
    eHatchStyle20Percent,                    // 8
    eHatchStyle25Percent,                    // 9
    eHatchStyle30Percent,                    // 10
    eHatchStyle40Percent,                    // 11
    eHatchStyle50Percent,                    // 12
    eHatchStyle60Percent,                    // 13
    eHatchStyle70Percent,                    // 14
    eHatchStyle75Percent,                    // 15
    eHatchStyle80Percent,                    // 16
    eHatchStyle90Percent,                    // 17
    eHatchStyleLightDownwardDiagonal,        // 18
    eHatchStyleLightUpwardDiagonal,          // 19
    eHatchStyleDarkDownwardDiagonal,         // 20
    eHatchStyleDarkUpwardDiagonal,           // 21
    eHatchStyleWideDownwardDiagonal,         // 22
    eHatchStyleWideUpwardDiagonal,           // 23
    eHatchStyleLightVertical,                // 24
    eHatchStyleLightHorizontal,              // 25
    eHatchStyleNarrowVertical,               // 26
    eHatchStyleNarrowHorizontal,             // 27
    eHatchStyleDarkVertical,                 // 28
    eHatchStyleDarkHorizontal,               // 29
    eHatchStyleDashedDownwardDiagonal,       // 30
    eHatchStyleDashedUpwardDiagonal,         // 31
    eHatchStyleDashedHorizontal,             // 32
    eHatchStyleDashedVertical,               // 33
    eHatchStyleSmallConfetti,                // 34
    eHatchStyleLargeConfetti,                // 35
    eHatchStyleZigZag,                       // 36
    eHatchStyleWave,                         // 37
    eHatchStyleDiagonalBrick,                // 38
    eHatchStyleHorizontalBrick,              // 39
    eHatchStyleWeave,                        // 40
    eHatchStylePlaid,                        // 41
    eHatchStyleDivot,                        // 42
    eHatchStyleDottedGrid,                   // 43
    eHatchStyleDottedDiamond,                // 44
    eHatchStyleShingle,                      // 45
    eHatchStyleTrellis,                      // 46
    eHatchStyleSphere,                       // 47
    eHatchStyleSmallGrid,                    // 48
    eHatchStyleSmallCheckerBoard,            // 49
    eHatchStyleLargeCheckerBoard,            // 50
    eHatchStyleOutlinedDiamond,              // 51
    eHatchStyleSolidDiamond,                 // 52

    eHatchStyleTotal,    // must be after all unique hatch styles

    eHatchStyleLargeGrid = eHatchStyleCross,  // 4  an alias for the cross style

    eHatchStyleMin       = eHatchStyleHorizontal,
    eHatchStyleMax       = eHatchStyleTotal - 1,
}GpHatchStyle;

//--------------------------------------------------------------------------
// Dash style constants
//--------------------------------------------------------------------------

typedef enum tagDashStyle
{
    eDashStyleSolid,          // 0
    eDashStyleDash,           // 1
    eDashStyleDot,            // 2
    eDashStyleDashDot,        // 3
    eDashStyleDashDotDot,     // 4
    eDashStyleCustom          // 5
}GpDashStyle;

//--------------------------------------------------------------------------
// Dash cap constants
//--------------------------------------------------------------------------

typedef enum tagDashCap
{
    eDashCapFlat             = 0,
    eDashCapRound            = 2,
    eDashCapTriangle         = 3
}GpDashCap;

//--------------------------------------------------------------------------
// Line cap constants (only the lowest 8 bits are used).
//--------------------------------------------------------------------------

typedef enum tagLineCap
{
    eLineCapFlat             = 0,
    eLineCapSquare           = 1,
    eLineCapRound            = 2,
    eLineCapTriangle         = 3,

    eLineCapNoAnchor         = 0x10, // corresponds to flat cap
    eLineCapSquareAnchor     = 0x11, // corresponds to square cap
    eLineCapRoundAnchor      = 0x12, // corresponds to round cap
    eLineCapDiamondAnchor    = 0x13, // corresponds to triangle cap
    eLineCapArrowAnchor      = 0x14, // no correspondence

    eLineCapCustom           = 0xff, // custom cap

    eLineCapAnchorMask       = 0xf0  // mask to check for anchor or not.
}GpLineCap;

//--------------------------------------------------------------------------
// Custom Line cap type constants
//--------------------------------------------------------------------------

typedef enum tagCustomLineCapType
{
    eCustomLineCapTypeDefault         = 0,
    eCustomLineCapTypeAdjustableArrow = 1
}GpCustomLineCapType;

//--------------------------------------------------------------------------
// Line join constants
//--------------------------------------------------------------------------

typedef enum tagLineJoin
{
    eLineJoinMiter        = 0,
    eLineJoinBevel        = 1,
    eLineJoinRound        = 2,
    eLineJoinMiterClipped = 3
}GpLineJoin;

//--------------------------------------------------------------------------
// Path point types (only the lowest 8 bits are used.)
//  The lowest 3 bits are interpreted as point type
//  The higher 5 bits are reserved for flags.
//--------------------------------------------------------------------------

typedef enum tagPathPointType
{
    ePathPointTypeStart           = 0,    // move
    ePathPointTypeLine            = 1,    // line
    ePathPointTypeBezier          = 3,    // default Beizer (= cubic Bezier)
    ePathPointTypePathTypeMask    = 0x07, // type mask (lowest 3 bits).
    ePathPointTypeDashMode        = 0x10, // currently in dash mode.
    ePathPointTypePathMarker      = 0x20, // a marker for the path.
    ePathPointTypeCloseSubpath    = 0x80, // closed flag

    // Path types used for advanced path.

    ePathPointTypeBezier2    = 2,    // quadratic Beizer
    ePathPointTypeBezier3    = 3,    // cubic Bezier
    ePathPointTypeBezier4    = 4,    // quartic (4th order) Beizer
    ePathPointTypeBezier5    = 5,    // quintic (5th order) Bezier
    ePathPointTypeBezier6    = 6     // hexaic (6th order) Bezier
}GpPathPointType;


//--------------------------------------------------------------------------
// WarpMode constants
//--------------------------------------------------------------------------

typedef enum tagWarpMode
{
    eWarpModePerspective,    // 0
    eWarpModeBilinear        // 1
}GpWarpMode;

//--------------------------------------------------------------------------
// LineGradient Mode
//--------------------------------------------------------------------------

typedef enum tagLinearGradientMode
{
    eLinearGradientModeHorizontal,         // 0
    eLinearGradientModeVertical,           // 1
    eLinearGradientModeForwardDiagonal,    // 2
    eLinearGradientModeBackwardDiagonal    // 3
}GpLinearGradientMode;

//--------------------------------------------------------------------------
// Region Comine Modes
//--------------------------------------------------------------------------

typedef enum tagCombineMode
{
    eCombineModeReplace,     // 0
    eCombineModeIntersect,   // 1
    eCombineModeUnion,       // 2
    eCombineModeXor,         // 3
    eCombineModeExclude,     // 4
    eCombineModeComplement   // 5 (does exclude from)
}GpCombineMode;

//--------------------------------------------------------------------------
 // Image types
//--------------------------------------------------------------------------

typedef enum tagImageType
{
    eImageTypeUnknown,   // 0
    eImageTypeBitmap,    // 1
    eImageTypeMetafile   // 2
}GpImageType;

//--------------------------------------------------------------------------
// Interpolation modes
//--------------------------------------------------------------------------

typedef enum tagInterpolationMode
{
    eInterpolationModeInvalid          = eQualityModeInvalid,
    eInterpolationModeDefault          = eQualityModeDefault,
    eInterpolationModeLowQuality       = eQualityModeLow,
    eInterpolationModeHighQuality      = eQualityModeHigh,
    eInterpolationModeBilinear,
    eInterpolationModeBicubic,
    eInterpolationModeNearestNeighbor,
    eInterpolationModeHighQualityBilinear,
    eInterpolationModeHighQualityBicubic
}GpInterpolationMode;

//--------------------------------------------------------------------------
// Pen types
//--------------------------------------------------------------------------
typedef enum tagPenAlignment
{
    ePenAlignmentCenter       = 0,
    ePenAlignmentInset        = 1,
    ePenAlignmentOutset       = 2,
    ePenAlignmentLeft         = 3,
    ePenAlignmentRight        = 4
}GpPenAlignment;

//--------------------------------------------------------------------------
// Brush types
//--------------------------------------------------------------------------

typedef enum tagBrushType
{
   eBrushTypeSolidColor       = 0,
   eBrushTypeHatchFill        = 1,
   eBrushTypeTextureFill      = 2,
   eBrushTypePathGradient     = 3,
   eBrushTypeLinearGradient   = 4
}GpBrushType;

//--------------------------------------------------------------------------
// Pen's Fill types
//--------------------------------------------------------------------------

typedef enum tagPenType
{
   ePenTypeSolidColor       = eBrushTypeSolidColor,
   ePenTypeHatchFill        = eBrushTypeHatchFill,
   ePenTypeTextureFill      = eBrushTypeTextureFill,
   ePenTypePathGradient     = eBrushTypePathGradient,
   ePenTypeLinearGradient   = eBrushTypeLinearGradient,
   ePenTypeUnknown          = -1
}GpPenType;

//--------------------------------------------------------------------------
// Matrix Order
//--------------------------------------------------------------------------

typedef enum tagMatrixOrder
{
    eMatrixOrderPrepend    = 0,
    eMatrixOrderAppend     = 1
}GpMatrixOrder;

//--------------------------------------------------------------------------
// Generic font families
//--------------------------------------------------------------------------

typedef enum tagGenericFontFamily
{
    eGenericFontFamilySerif,
    eGenericFontFamilySansSerif,
    eGenericFontFamilyMonospace

}GpGenericFontFamily;

//--------------------------------------------------------------------------
// FontStyle: face types and common styles
//--------------------------------------------------------------------------

//  These should probably be flags

//  Must have:
//      Regular = 0
//      Bold = 1
//      Italic = 2
//      BoldItalic = 3

typedef enum tagFontStyle
{
    eFontStyleRegular    = 0,
    eFontStyleBold       = 1,
    eFontStyleItalic     = 2,
    eFontStyleBoldItalic = 3,
    eFontStyleUnderline  = 4,
    eFontStyleStrikeout  = 8
}GpFontStyle;

//---------------------------------------------------------------------------
// Smoothing Mode
//---------------------------------------------------------------------------

typedef enum tagSmoothingMode
{
    eSmoothingModeInvalid     = eQualityModeInvalid,
    eSmoothingModeDefault     = eQualityModeDefault,
    eSmoothingModeHighSpeed   = eQualityModeLow,
    eSmoothingModeHighQuality = eQualityModeHigh,
    eSmoothingModeNone,
    eSmoothingModeAntiAlias
}GpSmoothingMode;

//---------------------------------------------------------------------------
// Pixel Format Mode
//---------------------------------------------------------------------------

typedef enum tagPixelOffsetMode
{
    ePixelOffsetModeInvalid     = eQualityModeInvalid,
    ePixelOffsetModeDefault     = eQualityModeDefault,
    ePixelOffsetModeHighSpeed   = eQualityModeLow,
    ePixelOffsetModeHighQuality = eQualityModeHigh,
    ePixelOffsetModeNone,    // no pixel offset
    ePixelOffsetModeHalf     // offset by -0.5, -0.5 for fast anti-alias perf
}GpPixelOffsetMode;

//---------------------------------------------------------------------------
// Text Rendering Hint
//---------------------------------------------------------------------------

typedef enum tagTextRenderingHint
{
    eTextRenderingHintSystemDefault = 0,            // Glyph with system default rendering hint
    eTextRenderingHintSingleBitPerPixelGridFit,     // Glyph bitmap with hinting
    eTextRenderingHintSingleBitPerPixel,            // Glyph bitmap without hinting
    eTextRenderingHintAntiAliasGridFit,             // Glyph anti-alias bitmap with hinting
    eTextRenderingHintAntiAlias,                    // Glyph anti-alias bitmap without hinting
    eTextRenderingHintClearTypeGridFit              // Glyph CT bitmap with hinting
}GpTextRenderingHint;

//---------------------------------------------------------------------------
// Metafile Types
//---------------------------------------------------------------------------
typedef enum tagMetafileType
{
    eMetafileTypeInvalid,            // Invalid metafile
    eMetafileTypeWmf,                // Standard WMF
    eMetafileTypeWmfAldus,           // Aldus Placeable Metafile format
    eMetafileTypeEmf,                // EMF (not EMF+)
    eMetafileTypeEmfPlusOnly,        // EMF+ without dual, down-level records
    eMetafileTypeEmfPlusDual         // EMF+ with dual, down-level records
}GpMetafileType;

// Specifies the type of EMF to record
typedef enum tagEmfType
{
    eEmfTypeEmfOnly     = eMetafileTypeEmf,          // no EMF+, only EMF
    eEmfTypeEmfPlusOnly = eMetafileTypeEmfPlusOnly,  // no EMF, only EMF+
    eEmfTypeEmfPlusDual = eMetafileTypeEmfPlusDual   // both EMF+ and EMF
}GpEmfType;

// All persistent objects must have a type listed here
typedef enum tagObjectType
{
    eObjectTypeInvalid,
    eObjectTypeBrush,
    eObjectTypePen,
    eObjectTypePath,
    eObjectTypeRegion,
    eObjectTypeImage,
    eObjectTypeFont,
    eObjectTypeStringFormat,
    eObjectTypeImageAttributes,
    eObjectTypeCustomLineCap,
    eObjectTypeMax = eObjectTypeCustomLineCap,
    eObjectTypeMin = eObjectTypeBrush
}GpObjectType;


//---------------------------------------------------------------------------
// EMF+ Records
//---------------------------------------------------------------------------

// We have to change the WMF record numbers so that they don't conflict with
// the EMF and EMF+ record numbers.

typedef int ieEmfPlusRecordType;
#define GP_EMFPLUS_RECORD_BASE        0x00004000
#define GP_WMF_RECORD_BASE            0x00010000
#define GP_WMF_RECORD_TO_EMFPLUS(n)   ((ieEmfPlusRecordType)((n) | GP_WMF_RECORD_BASE))
#define GP_EMFPLUS_RECORD_TO_WMF(n)   ((n) & (~GP_WMF_RECORD_BASE))
#define GP_IS_WMF_RECORDTYPE(n)       (((n) & GP_WMF_RECORD_BASE) != 0)

typedef enum tagEmfPlusRecordType
{
   // Since we have to enumerate GDI records right along with GDI+ records,
   // we list all the GDI records here so that they can be part of the
   // same enumeration type which is used in the enumeration callback.

    eWmfRecordTypeSetBkColor              = GP_WMF_RECORD_TO_EMFPLUS(META_SETBKCOLOR),
    eWmfRecordTypeSetBkMode               = GP_WMF_RECORD_TO_EMFPLUS(META_SETBKMODE),
    eWmfRecordTypeSetMapMode              = GP_WMF_RECORD_TO_EMFPLUS(META_SETMAPMODE),
    eWmfRecordTypeSetROP2                 = GP_WMF_RECORD_TO_EMFPLUS(META_SETROP2),
    eWmfRecordTypeSetRelAbs               = GP_WMF_RECORD_TO_EMFPLUS(META_SETRELABS),
    eWmfRecordTypeSetPolyFillMode         = GP_WMF_RECORD_TO_EMFPLUS(META_SETPOLYFILLMODE),
    eWmfRecordTypeSetStretchBltMode       = GP_WMF_RECORD_TO_EMFPLUS(META_SETSTRETCHBLTMODE),
    eWmfRecordTypeSetTextCharExtra        = GP_WMF_RECORD_TO_EMFPLUS(META_SETTEXTCHAREXTRA),
    eWmfRecordTypeSetTextColor            = GP_WMF_RECORD_TO_EMFPLUS(META_SETTEXTCOLOR),
    eWmfRecordTypeSetTextJustification    = GP_WMF_RECORD_TO_EMFPLUS(META_SETTEXTJUSTIFICATION),
    eWmfRecordTypeSetWindowOrg            = GP_WMF_RECORD_TO_EMFPLUS(META_SETWINDOWORG),
    eWmfRecordTypeSetWindowExt            = GP_WMF_RECORD_TO_EMFPLUS(META_SETWINDOWEXT),
    eWmfRecordTypeSetViewportOrg          = GP_WMF_RECORD_TO_EMFPLUS(META_SETVIEWPORTORG),
    eWmfRecordTypeSetViewportExt          = GP_WMF_RECORD_TO_EMFPLUS(META_SETVIEWPORTEXT),
    eWmfRecordTypeOffsetWindowOrg         = GP_WMF_RECORD_TO_EMFPLUS(META_OFFSETWINDOWORG),
    eWmfRecordTypeScaleWindowExt          = GP_WMF_RECORD_TO_EMFPLUS(META_SCALEWINDOWEXT),
    eWmfRecordTypeOffsetViewportOrg       = GP_WMF_RECORD_TO_EMFPLUS(META_OFFSETVIEWPORTORG),
    eWmfRecordTypeScaleViewportExt        = GP_WMF_RECORD_TO_EMFPLUS(META_SCALEVIEWPORTEXT),
    eWmfRecordTypeLineTo                  = GP_WMF_RECORD_TO_EMFPLUS(META_LINETO),
    eWmfRecordTypeMoveTo                  = GP_WMF_RECORD_TO_EMFPLUS(META_MOVETO),
    eWmfRecordTypeExcludeClipRect         = GP_WMF_RECORD_TO_EMFPLUS(META_EXCLUDECLIPRECT),
    eWmfRecordTypeIntersectClipRect       = GP_WMF_RECORD_TO_EMFPLUS(META_INTERSECTCLIPRECT),
    eWmfRecordTypeArc                     = GP_WMF_RECORD_TO_EMFPLUS(META_ARC),
    eWmfRecordTypeEllipse                 = GP_WMF_RECORD_TO_EMFPLUS(META_ELLIPSE),
    eWmfRecordTypeFloodFill               = GP_WMF_RECORD_TO_EMFPLUS(META_FLOODFILL),
    eWmfRecordTypePie                     = GP_WMF_RECORD_TO_EMFPLUS(META_PIE),
    eWmfRecordTypeRectangle               = GP_WMF_RECORD_TO_EMFPLUS(META_RECTANGLE),
    eWmfRecordTypeRoundRect               = GP_WMF_RECORD_TO_EMFPLUS(META_ROUNDRECT),
    eWmfRecordTypePatBlt                  = GP_WMF_RECORD_TO_EMFPLUS(META_PATBLT),
    eWmfRecordTypeSaveDC                  = GP_WMF_RECORD_TO_EMFPLUS(META_SAVEDC),
    eWmfRecordTypeSetPixel                = GP_WMF_RECORD_TO_EMFPLUS(META_SETPIXEL),
    eWmfRecordTypeOffsetClipRgn           = GP_WMF_RECORD_TO_EMFPLUS(META_OFFSETCLIPRGN),
    eWmfRecordTypeTextOut                 = GP_WMF_RECORD_TO_EMFPLUS(META_TEXTOUT),
    eWmfRecordTypeBitBlt                  = GP_WMF_RECORD_TO_EMFPLUS(META_BITBLT),
    eWmfRecordTypeStretchBlt              = GP_WMF_RECORD_TO_EMFPLUS(META_STRETCHBLT),
    eWmfRecordTypePolygon                 = GP_WMF_RECORD_TO_EMFPLUS(META_POLYGON),
    eWmfRecordTypePolyline                = GP_WMF_RECORD_TO_EMFPLUS(META_POLYLINE),
    eWmfRecordTypeEscape                  = GP_WMF_RECORD_TO_EMFPLUS(META_ESCAPE),
    eWmfRecordTypeRestoreDC               = GP_WMF_RECORD_TO_EMFPLUS(META_RESTOREDC),
    eWmfRecordTypeFillRegion              = GP_WMF_RECORD_TO_EMFPLUS(META_FILLREGION),
    eWmfRecordTypeFrameRegion             = GP_WMF_RECORD_TO_EMFPLUS(META_FRAMEREGION),
    eWmfRecordTypeInvertRegion            = GP_WMF_RECORD_TO_EMFPLUS(META_INVERTREGION),
    eWmfRecordTypePaintRegion             = GP_WMF_RECORD_TO_EMFPLUS(META_PAINTREGION),
    eWmfRecordTypeSelectClipRegion        = GP_WMF_RECORD_TO_EMFPLUS(META_SELECTCLIPREGION),
    eWmfRecordTypeSelectObject            = GP_WMF_RECORD_TO_EMFPLUS(META_SELECTOBJECT),
    eWmfRecordTypeSetTextAlign            = GP_WMF_RECORD_TO_EMFPLUS(META_SETTEXTALIGN),
    eWmfRecordTypeDrawText                = GP_WMF_RECORD_TO_EMFPLUS(0x062F),  // META_DRAWTEXT
    eWmfRecordTypeChord                   = GP_WMF_RECORD_TO_EMFPLUS(META_CHORD),
    eWmfRecordTypeSetMapperFlags          = GP_WMF_RECORD_TO_EMFPLUS(META_SETMAPPERFLAGS),
    eWmfRecordTypeExtTextOut              = GP_WMF_RECORD_TO_EMFPLUS(META_EXTTEXTOUT),
    eWmfRecordTypeSetDIBToDev             = GP_WMF_RECORD_TO_EMFPLUS(META_SETDIBTODEV),
    eWmfRecordTypeSelectPalette           = GP_WMF_RECORD_TO_EMFPLUS(META_SELECTPALETTE),
    eWmfRecordTypeRealizePalette          = GP_WMF_RECORD_TO_EMFPLUS(META_REALIZEPALETTE),
    eWmfRecordTypeAnimatePalette          = GP_WMF_RECORD_TO_EMFPLUS(META_ANIMATEPALETTE),
    eWmfRecordTypeSetPalEntries           = GP_WMF_RECORD_TO_EMFPLUS(META_SETPALENTRIES),
    eWmfRecordTypePolyPolygon             = GP_WMF_RECORD_TO_EMFPLUS(META_POLYPOLYGON),
    eWmfRecordTypeResizePalette           = GP_WMF_RECORD_TO_EMFPLUS(META_RESIZEPALETTE),
    eWmfRecordTypeDIBBitBlt               = GP_WMF_RECORD_TO_EMFPLUS(META_DIBBITBLT),
    eWmfRecordTypeDIBStretchBlt           = GP_WMF_RECORD_TO_EMFPLUS(META_DIBSTRETCHBLT),
    eWmfRecordTypeDIBCreatePatternBrush   = GP_WMF_RECORD_TO_EMFPLUS(META_DIBCREATEPATTERNBRUSH),
    eWmfRecordTypeStretchDIB              = GP_WMF_RECORD_TO_EMFPLUS(META_STRETCHDIB),
    eWmfRecordTypeExtFloodFill            = GP_WMF_RECORD_TO_EMFPLUS(META_EXTFLOODFILL),
    eWmfRecordTypeSetLayout               = GP_WMF_RECORD_TO_EMFPLUS(0x0149),  // META_SETLAYOUT
    eWmfRecordTypeResetDC                 = GP_WMF_RECORD_TO_EMFPLUS(0x014C),  // META_RESETDC
    eWmfRecordTypeStartDoc                = GP_WMF_RECORD_TO_EMFPLUS(0x014D),  // META_STARTDOC
    eWmfRecordTypeStartPage               = GP_WMF_RECORD_TO_EMFPLUS(0x004F),  // META_STARTPAGE
    eWmfRecordTypeEndPage                 = GP_WMF_RECORD_TO_EMFPLUS(0x0050),  // META_ENDPAGE
    eWmfRecordTypeAbortDoc                = GP_WMF_RECORD_TO_EMFPLUS(0x0052),  // META_ABORTDOC
    eWmfRecordTypeEndDoc                  = GP_WMF_RECORD_TO_EMFPLUS(0x005E),  // META_ENDDOC
    eWmfRecordTypeDeleteObject            = GP_WMF_RECORD_TO_EMFPLUS(META_DELETEOBJECT),
    eWmfRecordTypeCreatePalette           = GP_WMF_RECORD_TO_EMFPLUS(META_CREATEPALETTE),
    eWmfRecordTypeCreateBrush             = GP_WMF_RECORD_TO_EMFPLUS(0x00F8),  // META_CREATEBRUSH
    eWmfRecordTypeCreatePatternBrush      = GP_WMF_RECORD_TO_EMFPLUS(META_CREATEPATTERNBRUSH),
    eWmfRecordTypeCreatePenIndirect       = GP_WMF_RECORD_TO_EMFPLUS(META_CREATEPENINDIRECT),
    eWmfRecordTypeCreateFontIndirect      = GP_WMF_RECORD_TO_EMFPLUS(META_CREATEFONTINDIRECT),
    eWmfRecordTypeCreateBrushIndirect     = GP_WMF_RECORD_TO_EMFPLUS(META_CREATEBRUSHINDIRECT),
    eWmfRecordTypeCreateBitmapIndirect    = GP_WMF_RECORD_TO_EMFPLUS(0x02FD),  // META_CREATEBITMAPINDIRECT
    eWmfRecordTypeCreateBitmap            = GP_WMF_RECORD_TO_EMFPLUS(0x06FE),  // META_CREATEBITMAP
    eWmfRecordTypeCreateRegion            = GP_WMF_RECORD_TO_EMFPLUS(META_CREATEREGION),

    eEmfRecordTypeHeader                  = EMR_HEADER,
    eEmfRecordTypePolyBezier              = EMR_POLYBEZIER,
    eEmfRecordTypePolygon                 = EMR_POLYGON,
    eEmfRecordTypePolyline                = EMR_POLYLINE,
    eEmfRecordTypePolyBezierTo            = EMR_POLYBEZIERTO,
    eEmfRecordTypePolyLineTo              = EMR_POLYLINETO,
    eEmfRecordTypePolyPolyline            = EMR_POLYPOLYLINE,
    eEmfRecordTypePolyPolygon             = EMR_POLYPOLYGON,
    eEmfRecordTypeSetWindowExtEx          = EMR_SETWINDOWEXTEX,
    eEmfRecordTypeSetWindowOrgEx          = EMR_SETWINDOWORGEX,
    eEmfRecordTypeSetViewportExtEx        = EMR_SETVIEWPORTEXTEX,
    eEmfRecordTypeSetViewportOrgEx        = EMR_SETVIEWPORTORGEX,
    eEmfRecordTypeSetBrushOrgEx           = EMR_SETBRUSHORGEX,
    eEmfRecordTypeEOF                     = EMR_EOF,
    eEmfRecordTypeSetPixelV               = EMR_SETPIXELV,
    eEmfRecordTypeSetMapperFlags          = EMR_SETMAPPERFLAGS,
    eEmfRecordTypeSetMapMode              = EMR_SETMAPMODE,
    eEmfRecordTypeSetBkMode               = EMR_SETBKMODE,
    eEmfRecordTypeSetPolyFillMode         = EMR_SETPOLYFILLMODE,
    eEmfRecordTypeSetROP2                 = EMR_SETROP2,
    eEmfRecordTypeSetStretchBltMode       = EMR_SETSTRETCHBLTMODE,
    eEmfRecordTypeSetTextAlign            = EMR_SETTEXTALIGN,
    eEmfRecordTypeSetColorAdjustment      = EMR_SETCOLORADJUSTMENT,
    eEmfRecordTypeSetTextColor            = EMR_SETTEXTCOLOR,
    eEmfRecordTypeSetBkColor              = EMR_SETBKCOLOR,
    eEmfRecordTypeOffsetClipRgn           = EMR_OFFSETCLIPRGN,
    eEmfRecordTypeMoveToEx                = EMR_MOVETOEX,
    eEmfRecordTypeSetMetaRgn              = EMR_SETMETARGN,
    eEmfRecordTypeExcludeClipRect         = EMR_EXCLUDECLIPRECT,
    eEmfRecordTypeIntersectClipRect       = EMR_INTERSECTCLIPRECT,
    eEmfRecordTypeScaleViewportExtEx      = EMR_SCALEVIEWPORTEXTEX,
    eEmfRecordTypeScaleWindowExtEx        = EMR_SCALEWINDOWEXTEX,
    eEmfRecordTypeSaveDC                  = EMR_SAVEDC,
    eEmfRecordTypeRestoreDC               = EMR_RESTOREDC,
    eEmfRecordTypeSetWorldTransform       = EMR_SETWORLDTRANSFORM,
    eEmfRecordTypeModifyWorldTransform    = EMR_MODIFYWORLDTRANSFORM,
    eEmfRecordTypeSelectObject            = EMR_SELECTOBJECT,
    eEmfRecordTypeCreatePen               = EMR_CREATEPEN,
    eEmfRecordTypeCreateBrushIndirect     = EMR_CREATEBRUSHINDIRECT,
    eEmfRecordTypeDeleteObject            = EMR_DELETEOBJECT,
    eEmfRecordTypeAngleArc                = EMR_ANGLEARC,
    eEmfRecordTypeEllipse                 = EMR_ELLIPSE,
    eEmfRecordTypeRectangle               = EMR_RECTANGLE,
    eEmfRecordTypeRoundRect               = EMR_ROUNDRECT,
    eEmfRecordTypeArc                     = EMR_ARC,
    eEmfRecordTypeChord                   = EMR_CHORD,
    eEmfRecordTypePie                     = EMR_PIE,
    eEmfRecordTypeSelectPalette           = EMR_SELECTPALETTE,
    eEmfRecordTypeCreatePalette           = EMR_CREATEPALETTE,
    eEmfRecordTypeSetPaletteEntries       = EMR_SETPALETTEENTRIES,
    eEmfRecordTypeResizePalette           = EMR_RESIZEPALETTE,
    eEmfRecordTypeRealizePalette          = EMR_REALIZEPALETTE,
    eEmfRecordTypeExtFloodFill            = EMR_EXTFLOODFILL,
    eEmfRecordTypeLineTo                  = EMR_LINETO,
    eEmfRecordTypeArcTo                   = EMR_ARCTO,
    eEmfRecordTypePolyDraw                = EMR_POLYDRAW,
    eEmfRecordTypeSetArcDirection         = EMR_SETARCDIRECTION,
    eEmfRecordTypeSetMiterLimit           = EMR_SETMITERLIMIT,
    eEmfRecordTypeBeginPath               = EMR_BEGINPATH,
    eEmfRecordTypeEndPath                 = EMR_ENDPATH,
    eEmfRecordTypeCloseFigure             = EMR_CLOSEFIGURE,
    eEmfRecordTypeFillPath                = EMR_FILLPATH,
    eEmfRecordTypeStrokeAndFillPath       = EMR_STROKEANDFILLPATH,
    eEmfRecordTypeStrokePath              = EMR_STROKEPATH,
    eEmfRecordTypeFlattenPath             = EMR_FLATTENPATH,
    eEmfRecordTypeWidenPath               = EMR_WIDENPATH,
    eEmfRecordTypeSelectClipPath          = EMR_SELECTCLIPPATH,
    eEmfRecordTypeAbortPath               = EMR_ABORTPATH,
    eEmfRecordTypeReserved_069            = 69,  // Not Used
    eEmfRecordTypeGdiComment              = EMR_GDICOMMENT,
    eEmfRecordTypeFillRgn                 = EMR_FILLRGN,
    eEmfRecordTypeFrameRgn                = EMR_FRAMERGN,
    eEmfRecordTypeInvertRgn               = EMR_INVERTRGN,
    eEmfRecordTypePaintRgn                = EMR_PAINTRGN,
    eEmfRecordTypeExtSelectClipRgn        = EMR_EXTSELECTCLIPRGN,
    eEmfRecordTypeBitBlt                  = EMR_BITBLT,
    eEmfRecordTypeStretchBlt              = EMR_STRETCHBLT,
    eEmfRecordTypeMaskBlt                 = EMR_MASKBLT,
    eEmfRecordTypePlgBlt                  = EMR_PLGBLT,
    eEmfRecordTypeSetDIBitsToDevice       = EMR_SETDIBITSTODEVICE,
    eEmfRecordTypeStretchDIBits           = EMR_STRETCHDIBITS,
    eEmfRecordTypeExtCreateFontIndirect   = EMR_EXTCREATEFONTINDIRECTW,
    eEmfRecordTypeExtTextOutA             = EMR_EXTTEXTOUTA,
    eEmfRecordTypeExtTextOutW             = EMR_EXTTEXTOUTW,
    eEmfRecordTypePolyBezier16            = EMR_POLYBEZIER16,
    eEmfRecordTypePolygon16               = EMR_POLYGON16,
    eEmfRecordTypePolyline16              = EMR_POLYLINE16,
    eEmfRecordTypePolyBezierTo16          = EMR_POLYBEZIERTO16,
    eEmfRecordTypePolylineTo16            = EMR_POLYLINETO16,
    eEmfRecordTypePolyPolyline16          = EMR_POLYPOLYLINE16,
    eEmfRecordTypePolyPolygon16           = EMR_POLYPOLYGON16,
    eEmfRecordTypePolyDraw16              = EMR_POLYDRAW16,
    eEmfRecordTypeCreateMonoBrush         = EMR_CREATEMONOBRUSH,
    eEmfRecordTypeCreateDIBPatternBrushPt = EMR_CREATEDIBPATTERNBRUSHPT,
    eEmfRecordTypeExtCreatePen            = EMR_EXTCREATEPEN,
    eEmfRecordTypePolyTextOutA            = EMR_POLYTEXTOUTA,
    eEmfRecordTypePolyTextOutW            = EMR_POLYTEXTOUTW,
    eEmfRecordTypeSetICMMode              = 98,  // EMR_SETICMMODE,
    eEmfRecordTypeCreateColorSpace        = 99,  // EMR_CREATECOLORSPACE,
    eEmfRecordTypeSetColorSpace           = 100, // EMR_SETCOLORSPACE,
    eEmfRecordTypeDeleteColorSpace        = 101, // EMR_DELETECOLORSPACE,
    eEmfRecordTypeGLSRecord               = 102, // EMR_GLSRECORD,
    eEmfRecordTypeGLSBoundedRecord        = 103, // EMR_GLSBOUNDEDRECORD,
    eEmfRecordTypePixelFormat             = 104, // EMR_PIXELFORMAT,
    eEmfRecordTypeDrawEscape              = 105, // EMR_RESERVED_105,
    eEmfRecordTypeExtEscape               = 106, // EMR_RESERVED_106,
    eEmfRecordTypeStartDoc                = 107, // EMR_RESERVED_107,
    eEmfRecordTypeSmallTextOut            = 108, // EMR_RESERVED_108,
    eEmfRecordTypeForceUFIMapping         = 109, // EMR_RESERVED_109,
    eEmfRecordTypeNamedEscape             = 110, // EMR_RESERVED_110,
    eEmfRecordTypeColorCorrectPalette     = 111, // EMR_COLORCORRECTPALETTE,
    eEmfRecordTypeSetICMProfileA          = 112, // EMR_SETICMPROFILEA,
    eEmfRecordTypeSetICMProfileW          = 113, // EMR_SETICMPROFILEW,
    eEmfRecordTypeAlphaBlend              = 114, // EMR_ALPHABLEND,
    eEmfRecordTypeSetLayout               = 115, // EMR_SETLAYOUT,
    eEmfRecordTypeTransparentBlt          = 116, // EMR_TRANSPARENTBLT,
    eEmfRecordTypeReserved_117            = 117, // Not Used
    eEmfRecordTypeGradientFill            = 118, // EMR_GRADIENTFILL,
    eEmfRecordTypeSetLinkedUFIs           = 119, // EMR_RESERVED_119,
    eEmfRecordTypeSetTextJustification    = 120, // EMR_RESERVED_120,
    eEmfRecordTypeColorMatchToTargetW     = 121, // EMR_COLORMATCHTOTARGETW,
    eEmfRecordTypeCreateColorSpaceW       = 122, // EMR_CREATECOLORSPACEW,
    eEmfRecordTypeMax                     = 122,
    eEmfRecordTypeMin                     = 1,

    // That is the END of the GDI EMF records.

    // Now we start the list of EMF+ records.  We leave quite
    // a bit of room here for the addition of any new GDI
    // records that may be added later.

    eEmfPlusRecordTypeInvalid = GP_EMFPLUS_RECORD_BASE,
    eEmfPlusRecordTypeHeader,
    eEmfPlusRecordTypeEndOfFile,

    eEmfPlusRecordTypeComment,

    eEmfPlusRecordTypeGetDC,    // the application grabbed the metafile dc

    eEmfPlusRecordTypeMultiFormatStart,
    eEmfPlusRecordTypeMultiFormatSection,
    eEmfPlusRecordTypeMultiFormatEnd,

    // For all persistent objects
    eEmfPlusRecordTypeObject, // brush,pen,path,region,image,font,string-format

    // Drawing Records
    eEmfPlusRecordTypeClear,
    eEmfPlusRecordTypeFillRects,
    eEmfPlusRecordTypeDrawRects,
    eEmfPlusRecordTypeFillPolygon,
    eEmfPlusRecordTypeDrawLines,
    eEmfPlusRecordTypeFillEllipse,
    eEmfPlusRecordTypeDrawEllipse,
    eEmfPlusRecordTypeFillPie,
    eEmfPlusRecordTypeDrawPie,
    eEmfPlusRecordTypeDrawArc,
    eEmfPlusRecordTypeFillRegion,
    eEmfPlusRecordTypeFillPath,
    eEmfPlusRecordTypeDrawPath,
    eEmfPlusRecordTypeFillClosedCurve,
    eEmfPlusRecordTypeDrawClosedCurve,
    eEmfPlusRecordTypeDrawCurve,
    eEmfPlusRecordTypeDrawBeziers,
    eEmfPlusRecordTypeDrawImage,
    eEmfPlusRecordTypeDrawImagePoints,
    eEmfPlusRecordTypeDrawString,

    // Graphics State Records
    eEmfPlusRecordTypeSetRenderingOrigin,
    eEmfPlusRecordTypeSetAntiAliasMode,
    eEmfPlusRecordTypeSetTextRenderingHint,
    eEmfPlusRecordTypeSetTextContrast,
    eEmfPlusRecordTypeSetInterpolationMode,
    eEmfPlusRecordTypeSetPixelOffsetMode,
    eEmfPlusRecordTypeSetCompositingMode,
    eEmfPlusRecordTypeSetCompositingQuality,
    eEmfPlusRecordTypeSave,
    eEmfPlusRecordTypeRestore,
    eEmfPlusRecordTypeBeginContainer,
    eEmfPlusRecordTypeBeginContainerNoParams,
    eEmfPlusRecordTypeEndContainer,
    eEmfPlusRecordTypeSetWorldTransform,
    eEmfPlusRecordTypeResetWorldTransform,
    eEmfPlusRecordTypeMultiplyWorldTransform,
    eEmfPlusRecordTypeTranslateWorldTransform,
    eEmfPlusRecordTypeScaleWorldTransform,
    eEmfPlusRecordTypeRotateWorldTransform,
    eEmfPlusRecordTypeSetPageTransform,
    eEmfPlusRecordTypeResetClip,
    eEmfPlusRecordTypeSetClipRect,
    eEmfPlusRecordTypeSetClipPath,
    eEmfPlusRecordTypeSetClipRegion,
    eEmfPlusRecordTypeOffsetClip,

    eEmfPlusRecordTypeDrawDriverString,

    eEmfPlusRecordTotal,

    eEmfPlusRecordTypeMax = eEmfPlusRecordTotal-1,
    eEmfPlusRecordTypeMin = eEmfPlusRecordTypeHeader,
}GpEmfPlusRecordType;

//---------------------------------------------------------------------------
// StringFormatFlags
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// String format flags
//
//  DirectionRightToLeft          - For horizontal text, the reading order is
//                                  right to left. This value is called
//                                  the base embedding level by the Unicode
//                                  bidirectional engine.
//                                  For vertical text, columns are read from
//                                  right to left.
//                                  By default, horizontal or vertical text is
//                                  read from left to right.
//
//  DirectionVertical             - Individual lines of text are vertical. In
//                                  each line, characters progress from top to
//                                  bottom.
//                                  By default, lines of text are horizontal,
//                                  each new line below the previous line.
//
//  NoFitBlackBox                 - Allows parts of glyphs to overhang the
//                                  bounding rectangle.
//                                  By default glyphs are first aligned
//                                  inside the margines, then any glyphs which
//                                  still overhang the bounding box are
//                                  repositioned to avoid any overhang.
//                                  For example when an italic
//                                  lower case letter f in a font such as
//                                  Garamond is aligned at the far left of a
//                                  rectangle, the lower part of the f will
//                                  reach slightly further left than the left
//                                  edge of the rectangle. Setting this flag
//                                  will ensure the character aligns visually
//                                  with the lines above and below, but may
//                                  cause some pixels outside the formatting
//                                  rectangle to be clipped or painted.
//
//  DisplayFormatControl          - Causes control characters such as the
//                                  left-to-right mark to be shown in the
//                                  output with a representative glyph.
//
//  NoFontFallback                - Disables fallback to alternate fonts for
//                                  characters not supported in the requested
//                                  font. Any missing characters will be
//                                  be displayed with the fonts missing glyph,
//                                  usually an open square.
//
//  NoWrap                        - Disables wrapping of text between lines
//                                  when formatting within a rectangle.
//                                  NoWrap is implied when a point is passed
//                                  instead of a rectangle, or when the
//                                  specified rectangle has a zero line length.
//
//  NoClip                        - By default text is clipped to the
//                                  formatting rectangle. Setting NoClip
//                                  allows overhanging pixels to affect the
//                                  device outside the formatting rectangle.
//                                  Pixels at the end of the line may be
//                                  affected if the glyphs overhang their
//                                  cells, and either the NoFitBlackBox flag
//                                  has been set, or the glyph extends to far
//                                  to be fitted.
//                                  Pixels above/before the first line or
//                                  below/after the last line may be affected
//                                  if the glyphs extend beyond their cell
//                                  ascent / descent. This can occur rarely
//                                  with unusual diacritic mark combinations.

//---------------------------------------------------------------------------
typedef enum tagStringFormatFlags
{
    eStringFormatFlagsDirectionRightToLeft        = 0x00000001,
    eStringFormatFlagsDirectionVertical           = 0x00000002,
    eStringFormatFlagsNoFitBlackBox               = 0x00000004,
    eStringFormatFlagsDisplayFormatControl        = 0x00000020,
    eStringFormatFlagsNoFontFallback              = 0x00000400,
    eStringFormatFlagsMeasureTrailingSpaces       = 0x00000800,
    eStringFormatFlagsNoWrap                      = 0x00001000,
    eStringFormatFlagsLineLimit                   = 0x00002000,
    eStringFormatFlagsNoClip                      = 0x00004000
}GpStringFormatFlags;

//---------------------------------------------------------------------------
// StringTrimming
//---------------------------------------------------------------------------

typedef enum tagStringTrimming
{
    eStringTrimmingNone              = 0,
    eStringTrimmingCharacter         = 1,
    eStringTrimmingWord              = 2,
    eStringTrimmingEllipsisCharacter = 3,
    eStringTrimmingEllipsisWord      = 4,
    eStringTrimmingEllipsisPath      = 5
}GpStringTrimming;

//---------------------------------------------------------------------------
// National language digit substitution
//---------------------------------------------------------------------------

typedef enum tagStringDigitSubstitute
{
    eStringDigitSubstituteUser        = 0,  // As NLS setting
    eStringDigitSubstituteNone        = 1,
    eStringDigitSubstituteNational    = 2,
    eStringDigitSubstituteTraditional = 3
}GpStringDigitSubstitute;

//---------------------------------------------------------------------------
// Hotkey prefix interpretation
//---------------------------------------------------------------------------

typedef enum tagHotkeyPrefix
{
    eHotkeyPrefixNone        = 0,
    eHotkeyPrefixShow        = 1,
    eHotkeyPrefixHide        = 2
}GpHotkeyPrefix;

//---------------------------------------------------------------------------
// String alignment flags
//---------------------------------------------------------------------------

typedef enum tagStringAlignment
{
    // Left edge for left-to-right text,
    // right for right-to-left text,
    // and top for vertical
    eStringAlignmentNear   = 0,
    eStringAlignmentCenter = 1,
    eStringAlignmentFar    = 2
}GpStringAlignment;

//---------------------------------------------------------------------------
// DriverStringOptions
//---------------------------------------------------------------------------

typedef enum DriverStringOptions
{
    eDriverStringOptionsCmapLookup             = 1,
    eDriverStringOptionsVertical               = 2,
    eDriverStringOptionsRealizedAdvance        = 4,
    eDriverStringOptionsLimitSubpixel          = 8
}GpDriverStringOptions;

//---------------------------------------------------------------------------
// Flush Intention flags
//---------------------------------------------------------------------------

typedef enum tagFlushIntention
{
    eFlushIntentionFlush = 0,        // Flush all batched rendering operations
    eFlushIntentionSync = 1          // Flush all batched rendering operations
                                    // and wait for them to complete
}GpFlushIntention;

//---------------------------------------------------------------------------
// Image encoder parameter related types
//---------------------------------------------------------------------------

typedef enum tagEncoderParameterValueType
{
    eEncoderParameterValueTypeByte           = 1,    // 8-bit unsigned int
    eEncoderParameterValueTypeASCII          = 2,    // 8-bit byte containing one 7-bit ASCII
                                                    // code. NULL terminated.
    eEncoderParameterValueTypeShort          = 3,    // 16-bit unsigned int
    eEncoderParameterValueTypeLong           = 4,    // 32-bit unsigned int
    eEncoderParameterValueTypeRational       = 5,    // Two Longs. The first Long is the
                                                    // numerator, the second Long expresses the
                                                    // denomintor.
    eEncoderParameterValueTypeLongRange      = 6,    // Two longs which specify a range of
                                                    // integer values. The first Long specifies
                                                    // the lower end and the second one
                                                    // specifies the higher end. All values
                                                    // are inclusive at both ends
    eEncoderParameterValueTypeUndefined      = 7,    // 8-bit byte that can take any value
                                                    // depending on field definition
    eEncoderParameterValueTypeRationalRange  = 8     // Two Rationals. The first Rational
                                                    // specifies the lower end and the second
                                                    // specifies the higher end. All values
                                                    // are inclusive at both ends
}GpEncoderParameterValueType;

//---------------------------------------------------------------------------
// Image encoder value types
//---------------------------------------------------------------------------

typedef enum tagEncoderValue
{
    eEncoderValueColorTypeCMYK,
    eEncoderValueColorTypeYCCK,
    eEncoderValueCompressionLZW,
    eEncoderValueCompressionCCITT3,
    eEncoderValueCompressionCCITT4,
    eEncoderValueCompressionRle,
    eEncoderValueCompressionNone,
    eEncoderValueScanMethodInterlaced,
    eEncoderValueScanMethodNonInterlaced,
    eEncoderValueVersionGif87,
    eEncoderValueVersionGif89,
    eEncoderValueRenderProgressive,
    eEncoderValueRenderNonProgressive,
    eEncoderValueTransformRotate90,
    eEncoderValueTransformRotate180,
    eEncoderValueTransformRotate270,
    eEncoderValueTransformFlipHorizontal,
    eEncoderValueTransformFlipVertical,
    eEncoderValueMultiFrame,
    eEncoderValueLastFrame,
    eEncoderValueFlush,
    eEncoderValueFrameDimensionTime,
    eEncoderValueFrameDimensionResolution,
    eEncoderValueFrameDimensionPage
}GpEncoderValue;

//---------------------------------------------------------------------------
// Conversion of Emf To WMF Bits flags
//---------------------------------------------------------------------------

typedef enum tagEmfToWmfBitsFlags
{
    eEmfToWmfBitsFlagsDefault          = 0x00000000,
    eEmfToWmfBitsFlagsEmbedEmf         = 0x00000001,
    eEmfToWmfBitsFlagsIncludePlaceable = 0x00000002,
    eEmfToWmfBitsFlagsNoXORClip        = 0x00000004
}GpEmfToWmfBitsFlags;

//---------------------------------------------------------------------------
// Test Control flags
//---------------------------------------------------------------------------

typedef enum tagTestControlEnum
{
    eTestControlForceBilinear = 0,
    eTestControlNoICM = 1,
    eTestControlGetBuildNumber = 2
}GpTestControlEnum;


//---------------------------------------------------------------------------
// Window Change Notification types
//---------------------------------------------------------------------------
typedef enum tagGp_WindowNotifyEnum
{
   eWindowNotifyEnumEnable = 0,
   eWindowNotifyEnumDisable,
   eWindowNotifyEnumPalette,
   eWindowNotifyEnumDisplay,
   eWindowNotifyEnumSysColor
}GpWindowNotifyEnum;

typedef enum tagGp_ColorAdjustType
{
    eColorAdjustTypeDefault,
    eColorAdjustTypeBitmap,
    eColorAdjustTypeBrush,
    eColorAdjustTypePen,
    eColorAdjustTypeText,
    eColorAdjustTypeCount,   // must be immediately after all the individual ones
    eColorAdjustTypeAny      // internal use: for querying if any type has recoloring
}GpColorAdjustType;

//----------------------------------------------------------------------------
// Color Matrix flags
//----------------------------------------------------------------------------
typedef enum tagGp_ColorMatrixFlags
{
    eColorMatrixFlagsDefault   = 0,
    eColorMatrixFlagsSkipGrays = 1,
    eColorMatrixFlagsAltGray   = 2
}GpColorMatrixFlags;

//----------------------------------------------------------------------------
// Color Channel flags
//----------------------------------------------------------------------------
typedef enum tagColorChannelFlags
{
    eColorChannelFlagsC = 0,
    eColorChannelFlagsM,
    eColorChannelFlagsY,
    eColorChannelFlagsK,
    eColorChannelFlagsLast
}GpColorChannelFlags;


#endif // !_GDIPLUSENUMS_H
