/**************************************************************************\
*
* All files in GdiPlusLib have been modified from the Microsoft
* GdiPlus C++ header files.
*
* Module Name:
*
*   GdiPlusRegion.h
*
* Abstract:
*
*   Region related declarations
*
\**************************************************************************/

#ifndef _GDIPLUSREGION_H
#define _GDIPLUSREGION_H

#include "GdiPlusEnums.h"
#include "GdiPlusTypes.h"
#include "GdiPlusStubs.h"

typedef struct tagGp_Region
{
    GP_GPREGION * nativeRegion;
    GpStatus lastResult;
}GP_REGION;


//----------------------------------------------------------------------------
// Region methods
//----------------------------------------------------------------------------

GpStatus _stdcall GdipCreateRegion(GP_GPREGION **region);
GpStatus _stdcall GdipCreateRegionRect(const GPRECTF *rect, GP_GPREGION **region);
GpStatus _stdcall GdipCreateRegionRectI(const GPRECTI *rect, GP_GPREGION **region);
GpStatus _stdcall GdipCreateRegionPath(GP_GPPATH *path, GP_GPREGION **region);
GpStatus _stdcall GdipCreateRegionRgnData(const BYTE *regionData, INT size, GP_GPREGION **region);
GpStatus _stdcall GdipCreateRegionHrgn(HRGN hRgn, GP_GPREGION **region);
GpStatus _stdcall GdipCloneRegion(GP_GPREGION *region, GP_GPREGION **cloneRegion);
GpStatus _stdcall GdipDeleteRegion(GP_GPREGION *region);
GpStatus _stdcall GdipSetInfinite(GP_GPREGION *region);
GpStatus _stdcall GdipSetEmpty(GP_GPREGION *region);
GpStatus _stdcall GdipCombineRegionRect(GP_GPREGION *region, const GPRECTF *rect, GpCombineMode combineMode);
GpStatus _stdcall GdipCombineRegionRectI(GP_GPREGION *region, const GPRECTI *rect, GpCombineMode combineMode);
GpStatus _stdcall GdipCombineRegionPath(GP_GPREGION *region, GP_GPPATH *path, GpCombineMode combineMode);
GpStatus _stdcall GdipCombineRegionRegion(GP_GPREGION *region,  GP_GPREGION *region2, GpCombineMode combineMode);
GpStatus _stdcall GdipTranslateRegion(GP_GPREGION *region, REAL dx, REAL dy);
GpStatus _stdcall GdipTranslateRegionI(GP_GPREGION *region, INT dx, INT dy);
GpStatus _stdcall GdipTransformRegion(GP_GPREGION *region, GP_GPMATRIX *matrix);
GpStatus _stdcall GdipGetRegionBounds(GP_GPREGION *region, GP_GPGRAPHICS *graphics, GPRECTF *rect);
GpStatus _stdcall GdipGetRegionBoundsI(GP_GPREGION *region, GP_GPGRAPHICS *graphics, GPRECTI *rect);
GpStatus _stdcall GdipGetRegionHRgn(GP_GPREGION *region, GP_GPGRAPHICS *graphics, HRGN *hRgn);
GpStatus _stdcall GdipIsEmptyRegion(GP_GPREGION *region, GP_GPGRAPHICS *graphics, BOOL *result);
GpStatus _stdcall GdipIsInfiniteRegion(GP_GPREGION *region, GP_GPGRAPHICS *graphics, BOOL *result);
GpStatus _stdcall GdipIsEqualRegion(GP_GPREGION *region, GP_GPREGION *region2, GP_GPGRAPHICS *graphics, BOOL *result);
GpStatus _stdcall GdipGetRegionDataSize(GP_GPREGION *region, UINT * bufferSize);
GpStatus _stdcall GdipGetRegionData(GP_GPREGION *region, BYTE * buffer, UINT bufferSize, UINT * sizeFilled);
GpStatus _stdcall GdipIsVisibleRegionPoint(GP_GPREGION *region, REAL x, REAL y, GP_GPGRAPHICS *graphics, BOOL *result);
GpStatus _stdcall GdipIsVisibleRegionPointI(GP_GPREGION *region, INT x, INT y, GP_GPGRAPHICS *graphics, BOOL *result);
GpStatus _stdcall GdipIsVisibleRegionRect(GP_GPREGION *region, REAL x, REAL y, REAL width, REAL height, GP_GPGRAPHICS *graphics, BOOL *result);
GpStatus _stdcall GdipIsVisibleRegionRectI(GP_GPREGION *region, INT x, INT y, INT width, INT height, GP_GPGRAPHICS *graphics, BOOL *result);
GpStatus _stdcall GdipGetRegionScansCount(GP_GPREGION *region, UINT* count, GP_GPMATRIX* matrix);
GpStatus _stdcall GdipGetRegionScans(GP_GPREGION *region, GPRECTF* rects, INT* count, GP_GPMATRIX* matrix);
GpStatus _stdcall GdipGetRegionScansI(GP_GPREGION *region, GPRECTI* rects, INT* count, GP_GPMATRIX* matrix);

//-------------------------------------------------------
// GdiPlusLib.lib functions
//-------------------------------------------------------
GP_REGION GpRegion(VOID);
GP_REGION GpRegion_RcF(RECTF * rect);
GP_REGION GpRegion_RcI(const RECTI * rect);
GP_REGION GpRegion_GraphPath(const GP_GRAPHICSPATH* path);
GP_REGION GpRegion_RgnData(const BYTE* regionData, INT size);
GP_REGION GpRegion_Hrgn(HRGN hRgn);
VOID GpRegion_Delete(GP_REGION * this);
GP_REGION GpRegion_Clone(GP_REGION * this);
GpStatus GpRegion_MakeInfinite(GP_REGION * this);
GpStatus GpRegion_MakeEmpty(GP_REGION * this);
GpStatus GpRegion_Intersect(GP_REGION * this, const RECTF * rect);
GpStatus GpRegion_IntersectRcI(GP_REGION * this, const RECTI * rect);
GpStatus GpRegion_IntersectGraphPath(GP_REGION * this, const GP_GRAPHICSPATH* path);
GpStatus GpRegion_IntersectRegion(GP_REGION * this, const GP_REGION* region);
GpStatus GpRegion_UnionRcF(GP_REGION * this, const RECTF * rect);
GpStatus GpRegion_UnionRcI(GP_REGION * this, const RECTI * rect);
GpStatus GpRegion_UnionGraphPath(GP_REGION * this, const GP_GRAPHICSPATH* path);
GpStatus GpRegion_UnionRegion(GP_REGION * this, const GP_REGION* region);
GpStatus GpRegion_XorRcF(GP_REGION * this, const RECTF * rect);
GpStatus GpRegion_XorRcI(GP_REGION * this, const RECTI * rect);
GpStatus GpRegion_XorGraphPath(GP_REGION * this, const GP_GRAPHICSPATH* path);
GpStatus GpRegion_XorRegion(GP_REGION * this, const GP_REGION* region);
GpStatus GpRegion_ExcludeRcF(GP_REGION * this, const RECTF * rect);
GpStatus GpRegion_ExcludeRcI(GP_REGION * this, const RECTI * rect);
GpStatus GpRegion_ExcludeGraphPath(GP_REGION * this, const GP_GRAPHICSPATH* path);
GpStatus GpRegion_ExcludeRegion(GP_REGION * this, const GP_REGION* region);
GpStatus GpRegion_ComplementRcF(GP_REGION * this, const RECTF * rect);
GpStatus GpRegion_ComplementRcI(GP_REGION * this, const RECTI * rect);
GpStatus GpRegion_ComplementGraphPath(GP_REGION * this, const GP_GRAPHICSPATH* path);
GpStatus GpRegion_ComplementRegion(GP_REGION * this, const GP_REGION* region);
GpStatus GpRegion_TranslateF(GP_REGION * this, REAL dx, REAL dy);
GpStatus GpRegion_TranslateI(GP_REGION * this, INT dx, INT dy);
GpStatus GpRegion_Transform(GP_REGION * this, const GP_MATRIX* matrix);
GpStatus GpRegion_GetBoundsRcF(GP_REGION * this, RECTF* rect, const GP_GRAPHICS* g);
GpStatus GpRegion_GetBoundsRcI(GP_REGION * this, RECTI* rect, const GP_GRAPHICS* g);
HRGN GpRegion_GetHRGN(GP_REGION * this, const GP_GRAPHICS* g);
BOOL GpRegion_IsEmpty(GP_REGION * this, const GP_GRAPHICS *g);
BOOL GpRegion_IsInfinite(GP_REGION * this,  const GP_GRAPHICS *g);
BOOL GpRegion_Equals(GP_REGION * this, const GP_REGION* region, const GP_GRAPHICS* g);
UINT GpRegion_GetDataSize(GP_REGION * this);
GpStatus GpRegion_GetData(GP_REGION * this, BYTE* buffer, UINT bufferSize, UINT* sizeFilled);
BOOL GpRegion_IsVisiblePtF(GP_REGION * this, const POINTF * point,  const GP_GRAPHICS* g);
BOOL GpRegion_IsVisibleRcF(GP_REGION * this,  const RECTF * rect,  const GP_GRAPHICS* g);
BOOL GpRegion_IsVisiblePtI(GP_REGION * this, const POINTI * point, const GP_GRAPHICS* g);
BOOL GpRegion_IsVisibleRcI(GP_REGION * this, const RECTI * rect, const GP_GRAPHICS* g);
UINT GpRegion_GetRegionScansCount(GP_REGION * this, const GP_MATRIX* matrix);
GpStatus GpRegion_GetRegionScansRcF(GP_REGION * this, const GP_MATRIX* matrix, RECTF* rects, INT* count);
GpStatus GpRegion_GetRegionScansRcI(GP_REGION * this,  const GP_MATRIX* matrix, RECTI* rects, INT* count);

#endif // !_GDIPLUSREGION_H
