/**************************************************************************\
*
* All files in GdiPlusLib have been modified from the Microsoft
* GdiPlus C++ header files.
*
* Module Name:
*
*   GdiPlusStringFormat.h
*
* Abstract:
*
*   StringFormat related declarations
*
\**************************************************************************/

#ifndef _GDIPLUSSTRINGFORMAT_H
#define _GDIPLUSSTRINGFORMAT_H

#include "GdiPlusEnums.h"
#include "GdiPlusTypes.h"
#include "GdiPlusStubs.h"

typedef struct tagGp_StringFormat
{
    GP_GPSTRINGFORMAT * nativeFormat;
    GpStatus  lastResult;
}GP_STRINGFORMAT;

//----------------------------------------------------------------------------
// String DLL format API calls
//----------------------------------------------------------------------------

GpStatus _stdcall GdipCreateStringFormat(INT formatAttributes, LANGID language,
    GP_GPSTRINGFORMAT  **format);

GpStatus _stdcall GdipStringFormatGetGenericDefault(GP_GPSTRINGFORMAT **format);
GpStatus _stdcall GdipStringFormatGetGenericTypographic(GP_GPSTRINGFORMAT **format);
GpStatus _stdcall GdipDeleteStringFormat(GP_GPSTRINGFORMAT *format);
GpStatus _stdcall GdipCloneStringFormat(const GP_GPSTRINGFORMAT *format, GP_GPSTRINGFORMAT **newFormat);
GpStatus _stdcall GdipSetStringFormatFlags(GP_GPSTRINGFORMAT *format, INT flags);
GpStatus _stdcall GdipGetStringFormatFlags(const GP_GPSTRINGFORMAT *format, INT *flags);
GpStatus _stdcall GdipSetStringFormatAlign(GP_GPSTRINGFORMAT *format, GpStringAlignment align);
GpStatus _stdcall GdipGetStringFormatAlign(const GP_GPSTRINGFORMAT *format, GpStringAlignment *align);
GpStatus _stdcall GdipSetStringFormatLineAlign(GP_GPSTRINGFORMAT *format, GpStringAlignment align);
GpStatus _stdcall GdipGetStringFormatLineAlign(const GP_GPSTRINGFORMAT *format, GpStringAlignment *align);
GpStatus _stdcall GdipSetStringFormatTrimming(GP_GPSTRINGFORMAT *format, GpStringTrimming trimming);
GpStatus _stdcall GdipGetStringFormatTrimming(const GP_GPSTRINGFORMAT *format, GpStringTrimming *trimming);
GpStatus _stdcall GdipSetStringFormatHotkeyPrefix(GP_GPSTRINGFORMAT *format, INT hotkeyPrefix);
GpStatus _stdcall GdipGetStringFormatHotkeyPrefix(const GP_GPSTRINGFORMAT *format, INT *hotkeyPrefix);
GpStatus _stdcall GdipSetStringFormatTabStops(GP_GPSTRINGFORMAT *format, REAL firstTabOffset, INT count, const REAL *tabStops);
GpStatus _stdcall GdipGetStringFormatTabStops(const GP_GPSTRINGFORMAT *format, INT count, REAL *firstTabOffset, REAL *tabStops);
GpStatus _stdcall GdipGetStringFormatTabStopCount(const GP_GPSTRINGFORMAT *format, INT * count);

GpStatus _stdcall GdipDrawString(GP_GPGRAPHICS * graphics, const WCHAR *string,
	INT length, const GP_GPFONT * font, const RECTF * layoutRect,
    const GP_GPSTRINGFORMAT * stringFormat, const GP_GPBRUSH * brush);

GpStatus _stdcall GdipMeasureString(GP_GPGRAPHICS *graphics, const WCHAR *string,
    INT length, const GP_GPFONT *font, const RECTF  *layoutRect,
    const GP_GPSTRINGFORMAT *stringFormat, RECTF *boundingBox,
    INT *codepointsFitted, INT *linesFilled);

GpStatus _stdcall GdipMeasureCharacterRanges(GP_GPGRAPHICS *graphics,
    const WCHAR *string, INT length, const GP_GPFONT *font, const RECTF * layoutRect,
    const GP_GPSTRINGFORMAT *stringFormat, INT regionCount, GP_GPREGION **regions);

GpStatus _stdcall GdipDrawDriverString(GP_GPGRAPHICS *graphics, const UINT16 *text,
    INT length, const GP_GPFONT *font, const GP_GPBRUSH *brush,
    const POINTF *positions, INT flags, const GP_GPMATRIX *matrix);

GpStatus _stdcall GdipMeasureDriverString(GP_GPGRAPHICS *graphics, const UINT16 *text,
    INT length, const GP_GPFONT *font, const POINTF *positions, INT flags,
    const GP_GPMATRIX *matrix, RECTF *boundingBox);

GpStatus _stdcall GdipSetStringFormatDigitSubstitution(GP_GPSTRINGFORMAT *format, LANGID language, GpStringDigitSubstitute substitute);
GpStatus _stdcall GdipGetStringFormatDigitSubstitution(const GP_GPSTRINGFORMAT *format, LANGID *language, GpStringDigitSubstitute *substitute);

GpStatus _stdcall GdipGetStringFormatMeasurableCharacterRangeCount(const GP_GPSTRINGFORMAT *format, INT *count);
GpStatus _stdcall GdipSetStringFormatMeasurableCharacterRanges(GP_GPSTRINGFORMAT *format, INT rangeCount, const CHARACTERRANGE *ranges);

//-------------------------------------------------------
// GdiPlusLib.lib functions
//-------------------------------------------------------
GP_STRINGFORMAT GpStringFormatLANGID(INT formatFlags = 0, LANGID  language = LANG_NEUTRAL);
GP_STRINGFORMAT GpStringFormat(const GP_STRINGFORMAT *format);
GP_STRINGFORMAT GpStringFormat_Clone(GP_STRINGFORMAT * this);
VOID GpStringFormat_Delete(GP_STRINGFORMAT * this);
GpStatus GpStringFormat_SetFormatFlags(GP_STRINGFORMAT * this,INT flags);
INT GpStringFormat_GetFormatFlags(GP_STRINGFORMAT * this);
GpStatus GpStringFormat_SetAlignment(GP_STRINGFORMAT * this, GpStringAlignment align);
GpStringAlignment GpStringFormat_GetAlignment(GP_STRINGFORMAT * this);
GpStatus  GpStringFormat_SetLineAlignment(GP_STRINGFORMAT * this, GpStringAlignment align);
GpStringAlignment GpStringFormat_GetLineAlignment(GP_STRINGFORMAT * this);
GpStatus GpStringFormat_SetHotkeyPrefix(GP_STRINGFORMAT * this, GpHotkeyPrefix hotkeyPrefix);
GpHotkeyPrefix GpStringFormat_GetHotkeyPrefix(GP_STRINGFORMAT * this);
GpStatus GpStringFormat_SetTabStops(GP_STRINGFORMAT * this, REAL firstTabOffset, INT count, const REAL *tabStops);
INT GpStringFormat_GetTabStopCount(GP_STRINGFORMAT * this);
GpStatus GpStringFormat_GetTabStops(GP_STRINGFORMAT * this, INT count, REAL *firstTabOffset, REAL *tabStops);
GpStatus GpStringFormat_SetDigitSubstitution(GP_STRINGFORMAT * this, LANGID language, GpStringDigitSubstitute substitute);
LANGID GpStringFormat_GetDigitSubstitutionLanguage(GP_STRINGFORMAT * this);
GpStringDigitSubstitute GpStringFormat_GetDigitSubstitutionMethod(GP_STRINGFORMAT * this);
GpStatus GpStringFormat_SetTrimming(GP_STRINGFORMAT * this, GpStringTrimming trimming);
GpStringTrimming GpStringFormat_GetTrimming(GP_STRINGFORMAT * this);
GpStatus GpStringFormat_SetMeasurableCharacterRanges(GP_STRINGFORMAT * this, INT rangeCount, const CHARACTERRANGE *ranges);
INT GpStringFormat_GetMeasurableCharacterRangeCount(GP_STRINGFORMAT * this);
GpStatus GpStringFormat_GetLastStatus(GP_STRINGFORMAT * this);
GP_STRINGFORMAT GpStringFormat_StringFormat(GP_GPSTRINGFORMAT * clonedStringFormat, GpStatus status);
GP_STRINGFORMAT * GpStringFormat_GenericDefault(VOID);
const GP_STRINGFORMAT * GpStringFormat_GenericTypographic(VOID);


#endif // !_GDIPLUSSTRINGFORMAT_H
