/**************************************************************************\
*
* All files in GdiPlusLib have been modified from the Microsoft
* GdiPlus C++ header files.
*
* Module Name:
*
*   GplusTypes.h
*
* Abstract:
*
*   Basic types used by GdiPlus
*
\**************************************************************************/
#define WIN32_LEAN_AND_MEAN  /* speed up compilations */
#include <windows.h>
#include <float.h>

#ifndef _GDIPLUSTYPES_H
#define _GDIPLUSTYPES_H

#include "GdiPlusEnums.h"
#include "GdiPlusStubs.h"

typedef INT GpPixelFormat;
typedef struct IDirectDrawSurface7 IDirectDrawSurface7;

// The two memory DLL API calls
VOID * _stdcall GdipAlloc(size_t size);
VOID   _stdcall GdipFree(VOID * ptr);

#define GpAlloc GdipAlloc
#define GpFree GdipFree

typedef BOOL (CALLBACK * EnumerateMetafileProc)(GpEmfPlusRecordType,UINT,UINT,const BYTE*,VOID*);

//--------------------------------------------------------------------------
// Callback functions
//--------------------------------------------------------------------------

typedef BOOL (CALLBACK * ImageAbort)(VOID *);
typedef ImageAbort DrawImageAbort;
typedef ImageAbort GetThumbnailImageAbort;


// Callback for EnumerateMetafile methods.  The parameters are:

//      recordType      WMF, EMF, or EMF+ record type
//      flags           (always 0 for WMF/EMF records)
//      dataSize        size of the record data (in bytes), or 0 if no data
//      data            pointer to the record data, or NULL if no data
//      callbackData    pointer to callbackData, if any

// This method can then call Metafile::PlayRecord to play the
// record that was just enumerated.  If this method  returns
// FALSE, the enumeration process is aborted.  Otherwise, it continues.

typedef BOOL (CALLBACK * EnumerateMetafileProc)(GpEmfPlusRecordType,UINT,UINT,const BYTE*,VOID*);

//--------------------------------------------------------------------------
// Primitive data types
//
// NOTE:
//  Types already defined in standard header files:
//      INT8
//      UINT8
//      INT16
//      UINT16
//      INT32
//      UINT32
//      INT64
//      UINT64
//
//  Avoid using the following types:
//      LONG - use INT
//      ULONG - use UINT
//      DWORD - use UINT32
//--------------------------------------------------------------------------

typedef float REAL;
typedef DWORD ARGB;

#define REAL_MAX            FLT_MAX
#define REAL_MIN            FLT_MIN
#define REAL_TOLERANCE     (FLT_MIN * 100)
#define REAL_EPSILON        FLT_EPSILON       /*  1.192092896e-07F  */

//--------------------------------------------------------------------------
// Forward declarations of various internal classes
//--------------------------------------------------------------------------
typedef struct tagSize 			 SIZEI;
typedef struct tagSizeF 		 SIZEF;
typedef struct tagPoint 		 POINTI;
typedef struct tagPointF 		 POINTF;
typedef struct tagRect 			 RECTI;
typedef struct tagRectF 		 RECTF;
typedef struct tagCharacterRange CHARACTERRANGE;

//--------------------------------------------------------------------------
// Return values from any GDI+ API
//--------------------------------------------------------------------------
typedef POINTF GPPOINTF;
typedef POINTI GPPOINTI;
typedef RECTF GPRECTF;
typedef RECTI GPRECTI;
typedef SIZEF GPSIZEF;
typedef SIZEI GPSIZEI;

//--------------------------------------------------------------------------
// Represents a dimension in a 2D coordinate system
//  (floating-point coordinates)
//--------------------------------------------------------------------------
typedef struct tagSizeF
{   REAL Width;
    REAL Height;
}SIZEF;

//--------------------------------------------------------------------------
// Represents a dimension in a 2D coordinate system
//  (integer coordinates)
//--------------------------------------------------------------------------
typedef struct tagSize
{   int Width;
    int Height;
}GP_SIZEI;

//--------------------------------------------------------------------------
// Represents a location in a 2D coordinate system
//  (floating-point coordinates)
//--------------------------------------------------------------------------
typedef struct tagPointF
{   REAL X;
    REAL Y;
}POINTF;


//--------------------------------------------------------------------------
// Represents a location in a 2D coordinate system
//  (integer coordinates)
//--------------------------------------------------------------------------
typedef struct tagPoint
{   int X;
    int Y;
}POINTI;

//--------------------------------------------------------------------------
// Represents a rectangle in a 2D coordinate system
//  (floating-point coordinates)
//--------------------------------------------------------------------------
typedef struct tagRectF
{   REAL X;
    REAL Y;
    REAL Width;
    REAL Height;
}RECTF;

typedef struct tagRect
{
    int X;
    int Y;
    int Width;
    int Height;
}RECTI;

//-----------------------------
// text character range
//-----------------------------
typedef struct tagCharacterRange
{   int First;
    int Length;
}CHARACTERRANGE;

//----------------------------------------------------------------------------
// Color matrix
//----------------------------------------------------------------------------
typedef struct tagGp_ColorMatrix
{
    REAL m[5][5];
}COLORMATRIX;


typedef struct tagGp_Color
{
    ARGB Argb;
}COLOR;

//----------------------------------------------------------------------------
// Color Map
//----------------------------------------------------------------------------
typedef struct tagGp_ColorMap
{
    COLOR oldColor;
    COLOR newColor;
}COLOR_MAP; 
// COLORMAP would be in conflict with the COLORMAP in commctrl.h

//---------------------------------------------------------
// GdiPlusLib.lib functions
//---------------------------------------------------------
// SizeF
SIZEF   GpSizeF(REAL w, REAL h);
VOID 	GpSizeF_Cpy(SIZEF * this, SIZEF * size);
VOID 	GpSizeF_Set(SIZEF * this, REAL width, REAL height);
SIZEF   GpSizeF_Add(SIZEF * this, SIZEF * sz);
SIZEF   GpSizeF_Sub(SIZEF * this, SIZEF * sz);
BOOL 	GpSizeF_IsEqual(SIZEF * this, SIZEF * sz);
BOOL 	GpSizeF_IsEmpty(SIZEF * this);
// Size
GP_SIZEI GpSizeI(INT w, INT h);
VOID 	 GpSizeI_Cpy(GP_SIZEI * this, GP_SIZEI * sz);
VOID 	 GpSizeI_Set(GP_SIZEI * this, INT width, INT height);
GP_SIZEI GpSizeI_Add(GP_SIZEI * this, GP_SIZEI * sz);
GP_SIZEI GpSizeI_Sub(GP_SIZEI * this, GP_SIZEI * sz);
BOOL 	 GpSizeI_IsEqual(GP_SIZEI * this, GP_SIZEI * sz);
BOOL 	 GpSizeI_IsEmpty(GP_SIZEI * this);
// PointF
POINTF  GpPointF(REAL x, REAL y);
VOID 	GpPointF_Cpy(POINTF * this, POINTF * pointf);
VOID 	GpPointF_Size(POINTF * this, SIZEF * size);
VOID 	GpPointF_Set(POINTF * this, REAL x, REAL y);
POINTF  GpPointF_Add(POINTF * this, POINTF * pointf);
POINTF  GpPointF_Sub(POINTF * this, POINTF * pointf);
BOOL    GpPointF_IsEqual(POINTF * this, POINTF * pointf);
// Point
POINTI  GpPointI(INT x, INT y);
VOID 	GpPointI_Cpy(POINTI * this, POINTI * point);
VOID 	GpPointI_Size(POINTI * this, GP_SIZEI *size);
VOID 	GpPointI_Set(POINTI * this, INT x, INT y);
POINTI  GpPointI_Add(POINTI * this, POINTI * point);
POINTI  GpPointI_Sub(POINTI * this, POINTI * point);
BOOL 	GpPointI_IsEqual(POINTI * this, POINTI * point);
// RectF
RECTF   GpRectF(REAL x, REAL y, REAL width, REAL height);
VOID 	GpRectF_Set(RECTF * this, REAL x, REAL y, REAL width, REAL height);
RECTF   GpRectF_Clone(RECTF * this);
VOID 	GpRectF_GetLocation(RECTF * this, POINTF * pointf);
VOID 	GpRectF_GetSize(RECTF * this, SIZEF * size);
VOID 	GpRectF_GetBounds(RECTF * this, RECTF * rect);
REAL 	GpRectF_GetLeft(RECTF * this);
REAL 	GpRectF_GetTop(RECTF * this);
REAL 	GpRectF_GetRight(RECTF * this);
REAL 	GpRectF_GetBottom(RECTF * this);
BOOL 	GpRectF_IsEmptyArea(RECTF * this);
BOOL 	GpRectF_IsEqual(RECTF * this, RECTF * rect);
BOOL 	GpRectF_Contains(RECTF * this, REAL x, REAL y);
BOOL 	GpRectF_ContainsPtF(RECTF* this, const POINTF* pt);
VOID 	GpRectF_Inflate(RECTF * this, REAL dx, REAL dy);
BOOL 	GpRectF_Intersect(RECTF * c, RECTF * a, RECTF * b);
BOOL 	GpRectF_IntersectsWith(RECTF * this, RECTF* rect);
BOOL 	GpRectF_Union(RECTF * c, RECTF * a, RECTF * b);
VOID 	GpRectF_Offset(RECTF * this, REAL dx, REAL dy);
VOID 	GpRectF_OffsetRcF(RECTF* this, const RECTF* point);
// Rect
RECTI	GpRectI(INT x, INT y, INT width, INT height);
VOID 	GpRectI_Set(RECTI * this, INT x, INT y, INT width, INT height);
RECTI   GpRectI_Clone(RECTI * this);
VOID 	GpRectI_GetLocationPt(RECTI * this, POINTI * point);
VOID 	GpRectI_GetSize(RECTI * this, GP_SIZEI * size);
VOID 	GpRectI_GetBounds(RECTI * this, RECTI * rect);
REAL 	GpRectI_GetLeft(RECTI * this);
REAL 	GpRectI_GetTop(RECTI * this);
REAL 	GpRectI_GetRight(RECTI * this);
REAL 	GpRectI_GetBottom(RECTI * this);
BOOL 	GpRectI_IsEmptyArea(RECTI * this);
BOOL 	GpRectI_IsEqual(RECTI * this, RECTI * rect);
BOOL 	GpRectI_Contains(RECTI * this, INT x, INT y);
BOOL 	GpRectI_ContainsPt(RECTI * this, POINTI * pt);
VOID 	GpRectI_Inflate(RECTI * this, INT dx, INT dy);
BOOL 	GpRectI_Intersect(RECTI * c, RECTI * a, RECTI * b);
BOOL 	GpRectI_IntersectsWith(RECTI * this, RECTI * rect);
BOOL 	GpRectI_Union(RECTI * c, RECTI * a, RECTI * b);
VOID 	GpRectI_Offset(RECTI * this, INT dx, INT dy);
VOID 	GpRectI_OffsetPt(RECTI * this, POINTI * point);
BOOL 	ObjectTypeIsValid(GpObjectType type);

#endif // !_GDIPLUSTYPES_HPP

